const { editDocument } = require("application");
const { Color, Rectangle, Text, selection } = require("scenegraph");
const commands = require("commands");
const scenegraph = require("scenegraph");


const PANEL_HTML = `
<label><form id="formAddSquares"><button id="buttonAddSquares" type="submit" uxp-variant="cta">Add Squares</button></form></label>
<label><form id="formMakeCube"><button id="buttonMakeCube" type="submit" uxp-variant="cta">Make Cube</button></form></label>
<label><form id="formViewScenegraph"><button id="buttonViewScenegraph" type="submit" uxp-variant="cta">View Scenegraph</button></form></label>
<p><textarea id="textInfo" style="margin: -5px; overflow-y: auto; height: 70vh;" width="100%" readonly="true"></textarea></p>
`;  // PANEL_HTML


var _mainPanel;
var _indentLevel = 0;


/**
 * Creates the main panel if not already created, then returns it
 */
function getMainPanel() {
    _mainPanel = document.createElement("div");
    _mainPanel.innerHTML = PANEL_HTML;

    // Add button event listeners
    _mainPanel.querySelector("#formAddSquares").addEventListener("submit", handleAddSquares);
    _mainPanel.querySelector("#formMakeCube").addEventListener("submit", handleMakeCube);
    _mainPanel.querySelector("#formViewScenegraph").addEventListener("submit", handleViewScenegraph);

    return _mainPanel;
}  // getMainPanel



function getPadding() {
    let strPadding = "";

    for (let i = 0; i < _indentLevel; i++) {
        strPadding += "+";
    }

    return strPadding;
}


function getNodeInfo(nodeToInspect) {
    let strNodeInfo = "";

    if (nodeToInspect) {
        try {
            let strPadding = getPadding();
            strNodeInfo += strPadding + "Name: " + nodeToInspect.name.normalize() + "\n";
            strNodeInfo += strPadding + "color: " + JSON.stringify(nodeToInspect.fill) + "\n";
            strNodeInfo += strPadding + "width: " + nodeToInspect.globalBounds.width + "\n";
            strNodeInfo += strPadding + "height: " + nodeToInspect.globalBounds.height + "\n";
            strNodeInfo += strPadding + "X: " + nodeToInspect.globalBounds.x + "\n";
            strNodeInfo += strPadding + "Y: " + nodeToInspect.globalBounds.y + "\n";
            strNodeInfo += strPadding + "Z: " + nodeToInspect.zDepth + "\n";
            strNodeInfo += strPadding + "rotationX: " + nodeToInspect.rotationX + "\n";
            strNodeInfo += strPadding + "rotationY: " + nodeToInspect.rotationY + "\n";
            strNodeInfo += strPadding + "rotationZ: " + nodeToInspect.rotation + "\n";  // This is Z
            strNodeInfo += strPadding + "perspectiveCenterInParentCoordinates: " + JSON.stringify(nodeToInspect.perspectiveCenterInParentCoordinates) + "\n";

            if (nodeToInspect.constructor.name === "Text") {
                strNodeInfo += strPadding + "Text: " + nodeToInspect.text.normalize() + "\n";
            }

            strNodeInfo += "\n";
        } catch (err) {
            console.error(err);
        }
    }

    return strNodeInfo;
}  // getNodeInfo


function getNodeTreeRecursive(parentNode) {
    let strNodeTree = "";

    if (parentNode) {
        strNodeTree += getNodeInfo(parentNode);

        _indentLevel++;

        if (parentNode.children) {
            try {
                for (let i = 0; i < parentNode.children.length; i++) {
                    let childNode = parentNode.children.at(i);
                    strNodeTree += getNodeTreeRecursive(childNode);
                }
            } catch (err) {
                console.error(err);
            }
        }

        _indentLevel--;
    }

    return strNodeTree;
}  // getNodeTreeRecursive


/**
 * Returns a String containing the Scenegraph contents recursively
 */
function getScenegraph() {
    let strScenegraph = "";

    try {
        for (let i = 0; i < scenegraph.root.children.length; i++) {
            let childNode = scenegraph.root.children.at(i);
            _indentLevel = 0;
            strScenegraph += getNodeTreeRecursive(childNode);
        }
    } catch (err) {
        console.error(err);
    }

    return strScenegraph;
}  // getScenegraph


/**
 * Handle user pressing the "View Scenegraph" button
 */
async function handleViewScenegraph() {
    let strScenegraphAll = getScenegraph();
    setTextInfo(strScenegraphAll);
}  // handleViewScenegraph


/**
 * Handle user pressing the "Add Squares" button
 */
function handleAddSquares() {
    editDocument({ editLabel: "Adding shapes" }, addSquares);
}  // handleAddSquares


/**
 * Adds shapes to the parent of whatever is currently selected.  By default that will be the root of the current
 * canvas or artboard.  If a shape is selected, more shapes will be added to the selected shape's artboard.
 */
function addSquares() {
    let shapeWidth = 100;
    let shapeHeight = 100;
    let shapePadding = 20;
    // let squareColors = ["#FF595E", "#FFCA3A", "#8AC926", "#1982C4", "#6A4C93", "#676767"];  // Opaque
    let squareColors = [2164218206, 2164247098, 2156579110, 2149155524, 2154450067, 2151034928];  // With 50% alpha

    for (let shapeCounter = 0; shapeCounter < 6; shapeCounter++) {
        let shapeSquare = new Rectangle();
        shapeSquare.width = shapeWidth;
        shapeSquare.height = shapeHeight;
        shapeSquare.fill = new Color(squareColors[shapeCounter]);
        shapeSquare.stroke = new Color("Black");
        shapeSquare.strokeWidth = 2;
        selection.insertionParent.addChild(shapeSquare);
    }
}  // addSquares


/**
 * Repositions the 6 selected squares into a cube
 */
async function makeCube(squares) {
    setTextInfo("Cubing " + squares.length + " squares");

    squares[1].rotateYAround(90, squares[1].localCenterPoint);
    squares[2].rotateYAround(90, squares[2].localCenterPoint);
    squares[3].rotateXAround(90, squares[3].localCenterPoint);
    squares[4].rotateXAround(90, squares[4].localCenterPoint);

    squares[0].moveInParentCoordinates(100, 100, 0);
    squares[1].moveInParentCoordinates(50, 100, 50);
    squares[2].moveInParentCoordinates(150, 100, 50);
    squares[3].moveInParentCoordinates(100, 150, 50);
    squares[4].moveInParentCoordinates(100, 50, 50);
    squares[5].moveInParentCoordinates(100, 100);
    squares[5].zDepth = 100;

    commands.group();
}  // makeCube


/**
 *
 */
function handleMakeCube() {
    if (selection && selection.items && (selection.items.length > 0)) {
        editDocument({ editLabel: "Making cube" }, async function () { await makeCube(selection.items); });
    } else {
        setTextInfo("Select squares");
    }
}  // handleMakeCube


/**
 * Required handler for whenever the main panel is displayed
 */
function show(event) {
    if (!_mainPanel) {
        event.node.appendChild(getMainPanel());
    }
}  // show


/**
 * Sets the contents of the text area at the bottom of the control panel.
 * Can be set to empty.
 */
function setTextInfo(newText = "") {
    let textInfo = document.getElementById("textInfo");
    textInfo.value = newText;
}  // setTextInfo


/**
 * Required handler for whenever the user changes what is selected or modifies a shape
 */
function update(selection) {
}  // update


module.exports = {
    panels: {
        mainPanel: {
            show,
            update
        }
    }
};
